import rspolib

import pytest


def test_syntax_error(tests_dir):
    path = f"{tests_dir}/unescaped-double-quote-msgid.po"
    with pytest.raises(rspolib.SyntaxError, match="unescaped double quote found"):
        rspolib.pofile(path)


def test_io_error(tests_dir):
    path = f"{tests_dir}/invalid-version-number.mo"
    with pytest.raises(
        rspolib.IOError, match="Invalid mo file, expected revision number 0 or 1"
    ):
        rspolib.mofile(path)



import os
import sys

import pytest

ROOT_DIR = os.path.abspath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "..")
)

sys.path.append(os.path.join(ROOT_DIR, "python", "tests"))

from testing_utils import run_polibs


@pytest.fixture
def runner():
    return type("Polibs", (), {"run": run_polibs})


@pytest.fixture(autouse=True)
def tests_dir():
    return os.path.join(ROOT_DIR, "rust", "tests-data")


@pytest.fixture(autouse=True)
def output_dir():
    return os.path.join(ROOT_DIR, "python", "tests", "output")



def test_parse(runner, tests_dir):
    def parse_complete(polib):
        polib.pofile(f"{tests_dir}/django-complete.po")

    runner.run(
        parse_complete,
    )


def test_format(runner, tests_dir):
    import polib
    import rspolib

    rspo = rspolib.pofile(f"{tests_dir}/django-complete.po")
    pypo = polib.pofile(f"{tests_dir}/django-complete.po")

    def format_as_string(polib):
        assert (
            (rspo if polib.__name__ == "rspolib" else pypo)
            .__str__()
            .startswith("# This file is distributed")
        )

    runner.run(
        format_as_string,
    )


def test_edit_save(runner, tests_dir, output_dir):
    def edit_save(polib):
        po = polib.pofile(f"{tests_dir}/django-complete.po")
        po.metadata["Project-Id-Version"] = "test"
        entries = po.entries if polib.__name__ == "rspolib" else po
        for entry in entries:
            entry.msgstr = "test"
        po.save(f"{output_dir}/pofile_edit_save.po")
        po.save_as_mofile(f"{output_dir}/pofile_edit_save.mo")

    runner.run(
        edit_save,
    )


def test_methods(runner, tests_dir):
    def percent_translated(polib):
        po = polib.pofile(f"{tests_dir}/2-translated-entries.po")
        assert po.percent_translated() == 40

    def untranslated_entries(polib):
        po = polib.pofile(f"{tests_dir}/2-translated-entries.po")
        assert len(po.untranslated_entries()) == 3

    def translated_entries(polib):
        po = polib.pofile(f"{tests_dir}/2-translated-entries.po")
        assert len(po.translated_entries()) == 2

    def fuzzy_entries(polib):
        po = polib.pofile(f"{tests_dir}/fuzzy-no-fuzzy.po")
        assert len(po.fuzzy_entries()) == 1

    def find(polib):
        po = polib.pofile(f"{tests_dir}/flags.po")
        entry = po.find("msgstr 5", by="msgstr")
        if polib.__name__ == "rspolib":
            entry = entry[0]
        assert entry.msgid == "msgid 5"

    runner.run(
        percent_translated,
        untranslated_entries,
        translated_entries,
        fuzzy_entries,
        find,
    )


def test_find_entry(runner, tests_dir):
    import polib
    import rspolib

    pypo = polib.pofile(f"{tests_dir}/django-complete.po")
    rspo = rspolib.pofile(f"{tests_dir}/django-complete.po")

    def find_by_msgid(polib):
        if polib.__name__ == "rspolib":
            entry = rspo.find_by_msgid("Get started with Django")
        else:
            entry = pypo.find("Get started with Django")
        assert entry.msgstr == "Comienza con Django"

    def find_by_msgid_msgctxt(polib):
        if polib.__name__ == "rspolib":
            entry = rspo.find_by_msgid_msgctxt(
                "July",
                "abbrev. month",
            )
        else:
            entry = pypo.find(
                "July",
                msgctxt="abbrev. month",
            )
        assert entry.msgstr == "Jul."

    def find_by_msgid_plural(polib):
        if polib.__name__ == "rspolib":
            entries = rspo.find("Please submit %d or fewer forms.", by="msgid_plural")
            entry = entries[0]
        else:
            entry = pypo.find("Please submit %d or fewer forms.", by="msgid_plural")
        assert entry.msgstr_plural[0] == "Por favor, envíe %d formulario o menos."

    runner.run(
        find_by_msgid,
        find_by_msgid_msgctxt,
        find_by_msgid_plural,
    )


def test_magic_methods(runner, tests_dir):
    def iter__(polib):
        po = polib.pofile(f"{tests_dir}/django-complete.po")
        assert hasattr(po, "__iter__")

        iterated = False
        for entry in po:
            assert entry.msgid
            iterated = True
        assert iterated

    def len__(polib):
        po = polib.pofile(f"{tests_dir}/django-complete.po")
        assert hasattr(po, "__len__")
        assert len(po) > 320

    runner.run(
        iter__,
        len__,
    )



import time
import sys

import polib as pypolib
import rspolib

REPS = 50


def run_cb(cb, impl, polib, reps=REPS):
    start = time.time() * 1000
    for _ in range(reps):
        cb(polib)
    end = time.time() * 1000
    sys.stdout.write(f"{impl} {end - start} ms\n")


def run_polibs(*cbs, run_polib=True, run_rspolib=True, reps=REPS):
    for cb in cbs:
        sys.stdout.write("\n")
        sys.stdout.write(f"{cb.__name__}\n")
        if run_polib:
            run_cb(
                cb,
                "    polib",
                pypolib,
                reps=reps,
            )
        if run_rspolib:
            run_cb(
                cb,
                "  rspolib",
                rspolib,
                reps=reps,
            )



def test_parse(runner, tests_dir):
    def parse_all_features(polib):
        polib.mofile(f"{tests_dir}/all.mo")

    runner.run(
        parse_all_features,
    )


def test_format(runner, tests_dir):
    import polib
    import rspolib

    rspo = rspolib.mofile(f"{tests_dir}/all.mo")
    pypo = polib.mofile(f"{tests_dir}/all.mo")

    def format_as_string(polib):
        prefix = "#\n" if polib.__name__ == "rspolib" else ""
        assert (
            (rspo if polib.__name__ == "rspolib" else pypo)
            .__str__()
            .startswith(f'{prefix}msgid ""\n')
        )

    runner.run(
        format_as_string,
    )


def test_edit_save(runner, tests_dir, output_dir):
    def edit_save(polib):
        mo = polib.mofile(f"{tests_dir}/all.mo")
        mo.metadata["Project-Id-Version"] = "test"
        entries = mo.entries if polib.__name__ == "rspolib" else mo
        for entry in entries:
            entry.msgstr = "test"
        mo.save(f"{output_dir}/mofile_edit_save.mo")
        mo.save_as_pofile(f"{output_dir}/mofile_edit_save.po")

    runner.run(
        edit_save,
    )


def test_magic_methods(runner, tests_dir):
    def iter__(polib):
        po = polib.pofile(f"{tests_dir}/django-complete.po")
        assert hasattr(po, "__iter__")

        iterated = False
        for entry in po:
            assert entry.msgid
            iterated = True
        assert iterated

    def len__(polib):
        po = polib.pofile(f"{tests_dir}/django-complete.po")
        assert hasattr(po, "__len__")
        assert len(po) > 320

    runner.run(
        iter__,
        len__,
    )



def test_moentry_constructor(runner):
    def msgid_msgstr_kwargs(polib):
        entry = polib.MOEntry(
            msgid="msgid 1",
            msgstr="msgstr 1",
        )
        assert entry.msgid == "msgid 1"

    def msgid_plural(polib):
        entry = polib.MOEntry(
            msgid="msgid 1",
            msgid_plural="msgid_plural 1",
            msgstr_plural=[
                "msgstr_plural 1",
                "msgstr_plural 2",
            ],
        )
        assert entry.msgid == "msgid 1"
        assert entry.msgid_plural == "msgid_plural 1"
        assert entry.msgstr_plural == [
            "msgstr_plural 1",
            "msgstr_plural 2",
        ]

    def get_set_all(polib):
        entry = polib.MOEntry()
        entry.msgid = "msgid 1"
        entry.msgid_plural = "msgid_plural 1"
        entry.msgstr_plural = [
            "msgstr_plural 1",
            "msgstr_plural 2",
        ]
        entry.msgstr = "msgstr 1"
        entry.msgctxt = "msgctxt 1"
        assert entry.msgid == "msgid 1"
        assert entry.msgid_plural == "msgid_plural 1"
        assert entry.msgstr_plural == [
            "msgstr_plural 1",
            "msgstr_plural 2",
        ]
        assert entry.msgstr == "msgstr 1"
        assert entry.msgctxt == "msgctxt 1"

    runner.run(
        msgid_msgstr_kwargs,
        msgid_plural,
        get_set_all,
    )
